@@ -4,6 +4,8 @@
 #include <QStandardItemModel>
 #include <QMaemo5InformationBox>
 
+#include <errno.h>
+
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 
@@ -162,7 +164,7 @@ void MainWindow::setLayoutVB(const QString & keyboard)
 
 void MainWindow::setKbdModelVB(const QString & kbdModel)
 {
-    HalKeyboards halKeyboards;
+    HalKeyboards halKeyboards(this);
     QBTKeyboards btkeyboards(this);
 
     QStandardItemModel *model = new QStandardItemModel(this);
@@ -481,19 +483,64 @@ void MainWindow::makeFdi()
 
         QString options=kbdProps[5]+",caps:shiftlock";
 
-        file.write(QString(FDI_PRODUCT).arg(product).arg(kbdProps[0]).arg(layouts).arg(options).toAscii());
+	file.write(QString(FDI_PRODUCT).arg(product).arg(kbdProps[0]).arg(layouts).arg(options).toLatin1());
+	setXkbMap(product,kbdProps[0],layouts,options);
+
     }
     file.write(FDI_TRLR);
 }
 
+bool MainWindow::setXkbMap(const QString & product,const QString & model,const QString & layout, const QString & options)
+{
+    FILE * fp=popen(QString("hildon-im-xkbtool --list | grep \"%1\"").arg(product).toLatin1(),"r");
+    if(fp)
+    {
+	QString device;
+	QTextStream device_stream( fp, QIODevice::ReadOnly );
+	while((!(device = device_stream.readLine()).isNull()) && (!device.isEmpty()))
+	{
+	    QString device_id=device.split(",").value(0).split(" ").value(1);
+
+	    system(QString("setxkbmap "
+			   "-device \"%1\" "
+			   "-model  \"%2\" "
+			   "-layout \"%3\" "
+			   "-option \"%4\" "
+			   "-synch")
+			.arg(device_id)
+			.arg(model)
+			.arg(layout)
+			.arg(options)
+			.toLatin1()
+		   );
+	    QApplication::processEvents();
+
+	    system("setxkbmap");
+	    QApplication::processEvents();
+	}
+
+	pclose(fp);
+    }
+    else
+    {
+	QMaemo5InformationBox::information(this, QString("hildon-im-xkbtool error %1<br>%2").arg(errno).arg(strerror(errno)),
+						QMaemo5InformationBox::NoTimeout);
+	return false;
+    }
+    return true;
+}
+
 void MainWindow::on_buttonBox_accepted()
 {
-    //TODO save FDI here
+    QMaemo5InformationBox::information(this, QString(tr("Saving...")));
+    QApplication::processEvents();
+
     saveCurrentKeyboard();
+    QApplication::processEvents();
     makeFdi();
+    QApplication::processEvents();
     system("sudo /usr/libexec/extkbd-set");
-    QBTKeyboards kbds(this);
-    kbds.reconnect();
+    QApplication::processEvents();
 
     accept();
 }
